//
//  DemoViewController2.m
//  QCloudSDKDemo
//
//  Created by Sword on 2019/4/12.
//  Copyright © 2019 Tencent. All rights reserved.
//

#import "QDDemoViewController.h"
#import "QDOneSentenceRecognizeViewController.h"
#import "QDRealTimeRecognizeViewController.h"
#import "QDKeyboardSkipViewController.h"
#import "QDFileRecognizeViewController.h"
#import "ViewController.h"
#import "UIView+Toast.h"
#import "QDDefine.h"

#define kDemoListCellIdentifier @"kDemoListCellIdentifier"

@interface QDDemoViewController ()

@property (nonatomic, strong) NSArray *dataSource;

@end

@implementation QDDemoViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    [self.tableView registerClass:[UITableViewCell class] forCellReuseIdentifier:kDemoListCellIdentifier];
    _dataSource = @[@{@"text": @"一句话识别", @"class" : NSStringFromClass([QDOneSentenceRecognizeViewController class])},
                    @{@"text" : @"实时语音识别", @"class" : NSStringFromClass([QDRealTimeRecognizeViewController class])},
                    @{@"text" : @"录音文件识别", @"class" : NSStringFromClass([QDFileRecognizeViewController class])},
                    @{@"text" : @"语音合成", @"class" : NSStringFromClass([ViewController class])},
                    @{@"text" : @"实时语音识别关麦演示", @"class" : NSStringFromClass([QDKeyboardSkipViewController class])}
    ];
    self.tableView.tableFooterView = [self tableViewFooterView];
    // Uncomment the following line to preserve selection between presentations.
    // self.clearsSelectionOnViewWillAppear = NO;
    
    // Uncomment the following line to display an Edit button in the navigation bar for this view controller.
    // self.navigationItem.rightBarButtonItem = self.editButtonItem;
    
    [self checkQCloudAppInfo];
}

- (void)checkQCloudAppInfo
{
    if (![kQDAppId length] || ![kQDSecretId length] || ![kQDSecretKey length]) {
        [self.view makeToast:@"请设置 kQDAppId kQDSecretId kQDSecretKey" duration:1.3 position:CSToastPositionCenter];
    }
    
    dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(1.3 * NSEC_PER_SEC)), dispatch_get_main_queue(), ^{
        NSAssert([kQDAppId length] > 0, @"appid is not set");
        NSAssert([kQDSecretId length] > 0, @"secretId is not set");
        NSAssert([kQDSecretKey length] > 0, @"secretKey is not set");
    });
}


#pragma mark - Table view data source

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
    return 1;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return [_dataSource count];
}


- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:kDemoListCellIdentifier forIndexPath:indexPath];
    cell.textLabel.text = _dataSource[indexPath.row][@"text"];
    return cell;
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
    [tableView deselectRowAtIndexPath:indexPath animated:YES];
    
    if (indexPath.row == 3 || indexPath.row == 4) {
        NSString *classname = _dataSource[indexPath.row][@"class"];
        UIViewController *vc = [NSClassFromString(classname) new];
        [self.navigationController pushViewController:vc animated:YES];
        return;
    }
    UIViewController *vc = [self.storyboard instantiateViewControllerWithIdentifier:_dataSource[indexPath.row][@"class"]];
    [self.navigationController pushViewController:vc animated:YES];
}

- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender
{
    NSLog(@"seque %@", segue);
}

- (UIView *)tableViewFooterView {
    
    UIView *footerView = [[UIView alloc] initWithFrame:CGRectMake(0, 0, UIScreen.mainScreen.bounds.size.width, 100)];
    
    UIButton *checkButton = [[UIButton alloc] initWithFrame:CGRectMake(20, 20, 20, 20)];
    [checkButton setImage:[UIImage imageNamed:@"iconfinder_unchecked_checkbox"] forState:UIControlStateNormal];
    [checkButton setImage:[UIImage imageNamed:@"iconfinder_checked_checkbox"] forState:UIControlStateSelected];
    [checkButton addTarget:self action:@selector(checkClick:) forControlEvents:UIControlEventTouchUpInside];
    [footerView addSubview:checkButton];
    
    NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
    NSInteger authorize = [defaults integerForKey:@"authorize"];
    checkButton.selected = authorize;
    
    UILabel *tipLabel = [[UILabel alloc] initWithFrame:CGRectMake(50, 20, 100, 20)];
    tipLabel.font = [UIFont systemFontOfSize:12.0];
    tipLabel.text = @"按设备授权";
    [footerView addSubview:tipLabel];
    
    UILabel * label = [[UILabel alloc] initWithFrame:CGRectMake(UIScreen.mainScreen.bounds.size.width-200, 60, 180, 20)];
    label.textAlignment = NSTextAlignmentRight;
    label.font = [UIFont systemFontOfSize:12.0];
    label.text = [NSString stringWithFormat:@"sdk ver:%@", QCloudSDKShortVersionString];
    [footerView addSubview:label];
    return footerView;
}

- (void)checkClick:(UIButton *)sender {
    sender.selected = !sender.selected;
    NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
    [defaults setInteger:sender.selected forKey:@"authorize"];
    [defaults synchronize];
}
/*
// Override to support conditional editing of the table view.
- (BOOL)tableView:(UITableView *)tableView canEditRowAtIndexPath:(NSIndexPath *)indexPath {
    // Return NO if you do not want the specified item to be editable.
    return YES;
}
*/

/*
// Override to support editing the table view.
- (void)tableView:(UITableView *)tableView commitEditingStyle:(UITableViewCellEditingStyle)editingStyle forRowAtIndexPath:(NSIndexPath *)indexPath {
    if (editingStyle == UITableViewCellEditingStyleDelete) {
        // Delete the row from the data source
        [tableView deleteRowsAtIndexPaths:@[indexPath] withRowAnimation:UITableViewRowAnimationFade];
    } else if (editingStyle == UITableViewCellEditingStyleInsert) {
        // Create a new instance of the appropriate class, insert it into the array, and add a new row to the table view
    }   
}
*/

/*
// Override to support rearranging the table view.
- (void)tableView:(UITableView *)tableView moveRowAtIndexPath:(NSIndexPath *)fromIndexPath toIndexPath:(NSIndexPath *)toIndexPath {
}
*/

/*
// Override to support conditional rearranging of the table view.
- (BOOL)tableView:(UITableView *)tableView canMoveRowAtIndexPath:(NSIndexPath *)indexPath {
    // Return NO if you do not want the item to be re-orderable.
    return YES;
}
*/

/*
#pragma mark - Navigation

// In a storyboard-based application, you will often want to do a little preparation before navigation
- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender {
    // Get the new view controller using [segue destinationViewController].
    // Pass the selected object to the new view controller.
}
*/

@end
